#ifndef TOWN_H
#define TOWN_H

#include "Graph.h"
#include "Bar.h"
#include "Agent.h"
using namespace std;

//! \brief Town Class.
/*!
This class takes care of the initialization of all structures contained within a town (i.e.
the Agents within the population and Bar Array structures) deriving its parameters from the GUI
with which the client interacts and sets his/her desired simulation constraints.
*********************************************************************************************/
class Town{
public:

	/*! \brief Default Town Constructor
	*
	* Default constructor NOT utilized within code.
	******************************************************************************************/
	Town();
	
	/*! \brief Town Destructor
	*
	* Destructor NOT explicitly utilized within code.
	******************************************************************************************/
	~Town();

	/*! \brief Constructor used in the application. 
	*
	*\details Description: This is the backbone to initializing a town for a given simulation
	profile created via the GUI. Town Constructor which creates the Bar and Agent Objects using inputted 
	parameters by the createBars() and createAgents() functions.
	*
	*\param number_bars indicates how many bars will be available for simulated population to choose from.
	*\param population indicates how many Agents will be participating in the simulation.
	*\param user_cap[] the array which holds each bars respective capacity specified by the user through the GUI interface.
	*\param isPercent indicates whether the user inputted the bar capacities as a percentage of the population or just a Cap.
	*\param grpSize indicates the maximum size that a group within the population can be
	*\param avgAge2 indicates the Average Age at which an Agent expires
	*\param dropValue Indicates the threshold at which an Agent will generally drop a poorly performing Strategy and 
	* replace it with one of the top 20 performing Strategy's at random.
	*\pre The parameters are initialized within the following constraints: 
	0 \<  number_bars \< 256 &  0 \< population \< 16385 (2^14-1) and user_cap is the 
	array for the capacity for each bar. ( whose size may exceed the population.)
	
	*\post Barnums array and people array initialized as well as STM array
	which keeps track which bar has won.
	***********************************************************************/
	Town(int number_bars,int population,int user_cap[],bool isPercent,int grpSize,int avgAge2,int dropValue) );
	
	
	/*! \brief Initializes all bars to their respective "maxcapacity".
	*
	* \details Description: The Bar array barnums[] is initialized and bar[0] is the
	* normalizing home location that if the population is greater
	* than the sum of all "user_cap[]"s, then bar [0] gets the value of population - SUM(user_cap[]).
	* \param numbars indicates how many bars will be available for simulated population to choose from.
	* \param user_cap[] the array which holds each bars respective capacity specified by the user through the GUI interface.
	* \param isPercent indicates whether the user inputted the bar capacities as a percentage of the population or just a Cap.
	*
	* \pre 0 \< numbars \< 256
	* \post Bar array barnums[] has been initialized.

	**********************************************************************/
	void createBars(int numbars,int user_cap[],bool isPercent);
	
	/*! \brief Creates and initializes all Agents participating in a given simulation.
	* \details Description: Populates Agent array people[] with initialized Agent Objects.
	* \param population indicates how many Agents will be initialized for the simulation.
	* \param numGrps indicates how many groups will be initialized for the simulation.
	* \param avgAge indicates the average age that will be initialized for the simulation.
	* \param dropValue indicates the alpha value for score dropping that will be initialized for the simulation.
	* \pre 0 \< population \< 16385     (2^14-1)
	* \post Agent array people[] is initialized and holds all initialized Agents in the population.
	* \note By Default, if Group option is not selected there will only be one group that the entire 
	* population is a part of.
	**********************************************************************/
	void createGroups(int population,int numGrps,int avgAge,int dropValue);
	
	/*! \brief Simulates a turn in the Simulation.
	* \details Description: One turn is simulated when turn() is run. This includes polling all Agents
	* of which Bar they are going to through goingToBar(), deciding the winners of that particular turn through
	* getWinners(), and updates the statistics through tellWinners().
	*
	* \pre Input parameters have been passed from the GUI and have initialized all appropriate instances.
	* \post One turn has passed, all parameters have been updated and recorded.
	* \return graphPtr object which contains all information for the past turn. 
	*********************************************************************/
	graphPtr turn();
	
	/*! \brief Polls all Agents to get their consensus for a given turn.
	* \details Description: Polls all the Agents participating in the simulation of which bar they
	* are going to and tallies the Agents answers for each bar. A pointer to an Array is returned which contains
	* this information.
	* \pre All necessary variables are set and a simulation is underway.
	* \post Returns a pointer to an array which holds counters for how many
	* Agents are going to a particular bar for a given turn.
	* \return Integer pointer to array which contains the tally of how many Agents went to each bar for a given turn.
	***********************************************************************/
	int* goingToBar();
	
	/*! \brief Calculates winners for a given round.
	* \details Description: Function updates the STM to add current turn data and also updates other optional
	* statistics such as how many bars won and how many Agents won for the current round. Also populates
	* Stuff structure to pass back to the GUI for graph output.
	* \param a[256] structure that holds information about which bar has won the current round
	* \pre Input is an int array which contains 1s's where bars won 
	* and 0's where bars lost
	* \post Bars that have won for a particular round have been acknowledged
	* and updated
	* \return Integer pointer to array which contains information whether a given instialized bar has won that round.
	*********************************************************************/
	int* getWinners(int a[256]);
	
	/*! \brief Converts statistics into visually respresentable data for GUI to utilize.
	* \details Description: Function goes through the population and tells an agent they have either won or
	* lost the current round which depends if the bar has won or lost that round. Also populates Stuff
	* structure to pass back to the GUI for graph output.
	* \param a[256] structure that holds information about which bar has won the current round
	* \pre The array that holds valid bars for the current simulation has been populated.
	* \post Stuff sructure has been populated.
	************************************************************************/
	void tellWinners(int a[256]);

	
	/*! \brief Controls Population
	* \details Description: Controls the dynamic Agent Population by keeping track of the groups within
	* the population. While a group is not full, new Agents are created to fill this gap.
	* \pre All necessary variables are set and a simulation is underway. At least one group within the 
	* population is NOT at maximum occupancy.
	* \post Every group has maximum occupancy.
	***********************************************************************/
	void Town::popcontrol();

protected:
	
	/*! \brief Number of Bars in Simulations
       * Reference variable that is used for loops to iterate through all active bars in simulation.
       */
	int numBars;
	
	/*! \brief Population in Simulation
       * Reference variable that is used for loops to iterate through all Agents that are in the simulation profile
	   * \note Will be constant unless simulation option chosen to have deaths and births.
       */
	int numPeeps;
	
	/*! \brief Bar holder array
       * Array structure used for keeping track of status of a bar ( i.e. won/lost)
       */
	Bar barNums[256];
	
	/*! \brief Agent holder array
       * Array structure that holds all Agent objects for the simulation
       */
	Agent people[16384];
	
	/*! \brief Short Term Memory
       * Pointer to structure that keeps track of recent winnners in the simulation.
       */
	int *STM;
	
	/*! \brief Connecting Medium
       * The connecting structure medium that allows communication between the frontend GUI interface and
	   * computational backend. Allows the update of statistics to applied to graphs and recording mechanism for
	   * later review.
       */
	graphPtr Stuff;
	
	/*! \brief Group Size
       * Variable that holds the max size that a group can be.
       */
	int grpSize;

	/*! \brief Number of Agnets
       * Variable that holds population size.
       */
	int numpeeps;
	
	/*! \brief Alpha Value
       * Variable that holds the alpha value which represents the threshold in which strategies are dropped inside groups.
       */
	int drop;
	
	/*! \brief Bar Array
       * A double pointer holding all bars and how many people went to each bar.
       */
	Bar ** barnums;
	
	/*! \brief Groups
       * List that holds all groups.
       */
	list<Group *> groups;
	
	/*! \brief Group Iterator
       * Iterator for group list.
       */
	list<Group *>::iterator it;
	
	/*! \brief Top Strategies
       * Double pointer holding the top strategies.
       */
	Strategy ** topStrats;
	
	/*! \brief Short Term Index
       * The index of which strategy is being used.
       */
	int STI;
	
	/*! \brief Average Age
       * Variable holds the average age.
       */
	int avgAge;
};

#endif